# ci_prop_diff_mn matches the values in the paper

    Code
      ci_prop_diff_mn(response, treat, data = df)
    Message
      
      -- Miettinen-Nurminen Confidence Interval --------------------------------------
      * 10/10 - 0/20
      * Estimate: 1
      * 95% Confidence Interval:
        (0.7156, 1)

